local SCALE = 50
local NUMBALLS = 600
local BALLRADIUS = 8
local UNMARKED, MARKED = { .5, .5, .6, 1 }, { 0, .7, 1, 1 }
local OBSX, OBSY, OBSR = 400, 200, 50

local phy, gfx = love.physics, love.graphics
local rng = love.math.newRandomGenerator(12345)

local MARK = {}
for _, i in ipairs({
	1, 27, 28, 37, 47, 52, 69, 70, 73, 84, 92, 98, 99, 127, 129, 131, 137, 141, 145, 146, 150, 156, 159, 160, 162, 163,
	165, 168, 176, 180, 181, 182, 187, 192, 198, 201, 221, 226, 242, 244, 248, 252, 258, 260, 266, 267, 279, 280, 286,
	290, 297, 305, 309, 310, 317, 323, 328, 332, 351, 355, 358, 370, 382, 385, 394, 396, 398, 410, 418, 425, 427, 434,
	437, 444, 448, 453, 460, 462, 470, 472, 473, 475, 484, 489, 495, 501, 505, 511, 516, 521, 535, 549, 550, 551, 552,
	553, 554, 569, 570, 580, 600
}) do
	MARK[i] = MARKED
end

phy.setMeter(SCALE)
local world = phy.newWorld(0, SCALE * 9.81, true)

phy.newFixture(phy.newBody(world,    0,  600, 'static'), phy.newEdgeShape(0, 0, 800, 0))
phy.newFixture(phy.newBody(world,    0,    0, 'static'), phy.newEdgeShape(0, 0, 0, 600))
phy.newFixture(phy.newBody(world,  800,    0, 'static'), phy.newEdgeShape(0, 0, 0, 600))
phy.newFixture(phy.newBody(world, OBSX, OBSY, 'static'), phy.newCircleShape(OBSR))

local ballShape = phy.newCircleShape(BALLRADIUS)
local balls = {}

local function nextBall()
	local x, y = (400 - BALLRADIUS) + rng:random(BALLRADIUS * 2), -rng:random(BALLRADIUS * 4)
	local body = phy.newBody(world, x, y, 'dynamic')
	local fix = phy.newFixture(body, ballShape)
	fix:setRestitution(.7)
	table.insert(balls, { body, MARK[#balls + 1] or UNMARKED })
end

local accum = 0
function love.update(dt)
	accum = accum + math.min(dt, 1 / 20)
	while accum >= 1 / 60 do
		world:update(1 / 60)
		accum = accum - 1 / 60
		if #balls < NUMBALLS then
			nextBall()
		end
	end
end

local sprite = gfx.newCanvas(BALLRADIUS * 2, BALLRADIUS * 2)
sprite:renderTo(function()
	gfx.circle('fill', BALLRADIUS, BALLRADIUS, BALLRADIUS)
	gfx.circle('line', BALLRADIUS, BALLRADIUS, BALLRADIUS)
end)

gfx.setBackgroundColor(.2, .2, .2)
gfx.setBlendMode('alpha', 'premultiplied')

function love.draw()
	gfx.origin()
	local scale = math.min(gfx.getWidth() / 800, gfx.getHeight() / 600)
	gfx.translate(math.floor(.5 * (gfx.getWidth() - scale * 800)),  math.floor(.5 * (gfx.getHeight() - scale * 600)))
	gfx.scale(scale)

	gfx.setColor(0, 0, 0)
	gfx.circle('fill', OBSX, OBSY, OBSR)

	for _, ball in ipairs(balls) do
		local body, color = ball[1], ball[2]
		local x, y = body:getPosition()
		gfx.setColor(color)
		gfx.draw(sprite, x, y, 0, 1, 1, BALLRADIUS, BALLRADIUS)
	end
end
